/*
 * Copyright 2016 Daniel Skogquist Åborg
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.d2ab.function;

import java.util.function.Consumer;
import java.util.function.IntConsumer;

/**
 * A specialization of {@link Consumer} that accepts {@code char} values. Adapted from {@link IntConsumer} and the like.
 */
@FunctionalInterface
public interface CharConsumer {
	/**
	 * Perform this action on the given value.
	 */
	void accept(char c);

	/**
	 * Combine this consumer with another consumer whose action is always performed after this consumer's action.
	 */
	default CharConsumer andThen(CharConsumer consumer) {
		return t -> {
			accept(t);
			consumer.accept(t);
		};
	}
}
